import json

from starfish.utils.crypto_hash import hash_sha3_256
from invokable.constants import METADATA_ADDITIONAL_INFORMATION_NAME


class InvokeItem:

    def __init__(self, name, invoke_class, metadata_text, target=None, asset_id=None):
        self._name = name
        self._invoke_class = invoke_class
        self._metadata_text = metadata_text
        self._asset_id = asset_id
        if self._asset_id is None:
            self._asset_id = hash_sha3_256(metadata_text)

        self._target = target

    @property
    def name(self):
        return self._name

    @property
    def asset_id(self):
        return self._asset_id

    @property
    def invoke_class(self):
        return self._invoke_class

    @property
    def target(self):
        return self._target

    @property
    def metadata_text(self):
        """

        Get a orginal metadata as a string, based on the service name.

        """
        return self._metadata_text

    @property
    def metadata(self):
        """

        Get the metadata as a dict


        """
        return json.loads(self._metadata_text)

    @property
    def metadata_additional(self):
        """

        Get additioanal metadata saved by this library.
        Since the current standard does not allow for different data types, this
        library saves the actual metadata parameter and results definitions
        as an additional items in the metadata.

        You can use this method to get the additional parameters and results definitions.

        :return: Additional data from this library

        :type: dict or None

        """
        metadata = json.loads(self._metadata_text)
        if metadata and metadata['additionalInformation'] \
                and METADATA_ADDITIONAL_INFORMATION_NAME in metadata['additionalInformation']:
            return metadata['additionalInformation'][METADATA_ADDITIONAL_INFORMATION_NAME]
        return None
