"""

Metadata Validator


"""

import logging
import yaml

from jsonschema import validate

logger = logging.getLogger('invokable.metadata_tools')


OPERATION_SCHEMA_0_0_1 = """
type: object
required: [name, type, operation]
additionalProperties: false
properties:
  name:
    type: string
  type:
    type: string
    enum: [dataset, operation, bundle]
  description:
    type: string
  dateCreated:
    type: string
  creator:
    type: string
  license:
    type: string
  copyrightHolder:
    type: string
  links:
    type: array
    items:
      type: object
  inLanguage:
    type: string
  tags:
    type: array
    items:
      type: string
  additionalInformation:
    type: [object, array]

  operation:
    type: object
    required: [inputs, outputs]
    properties:
      class:
        type: string
      modes:
        type: array
        items:
          type: string
          enum: [async, sync]
      inputs:
        type: ['null', object]
        patternProperties:
          '^.*$':
            type: object
            required: [type]
            properties:
              type:
                type: string
                enum: [asset, json]
              position:
                type: integer
              required:
                type: boolean
        additionalProperties: False
      outputs:
        type: ['null', object]
  contentHash:
    type: string
"""


def metadata_validator(metadata):

    schema = yaml.safe_load(OPERATION_SCHEMA_0_0_1)
    validate(instance=metadata, schema=schema)
    return True
