
import os
import sqlite3

from invokable.storage.storage import Storage


class StorageSqlite(Storage):

    ALLOWED_NAMES = [
        'metadata',
        'asset'
    ]
    def __init__(self, filename):

        self._filename = filename
        self._table_names = []

    def items(self, name):
        result = {}
        db, cursor = self._connect(name)
        cursor.execute(f'SELECT asset_id, data FROM {name}')
        for row in cursor.fetchmany():
            result[row[0]] = row[1]
        db.close()
        return result

    def read(self, name, asset_id):
        db, cursor = self._connect(name)
        cursor.execute(f'SELECT data FROM {name} WHERE asset_id = ?', (asset_id,))
        row = cursor.fetchone()
        data = row[0]
        db.close()
        return data

    def write(self, name, asset_id, data):
        db, cursor = self._connect(name)
        cursor.execute(f'SELECT data FROM {name} WHERE asset_id = ?', (asset_id,))
        row = cursor.fetchone()
        if row:
            cursor.execute(f'UPDATE {name} SET data = ? WHERE asset_id = ?', (data, asset_id))
        else:
            cursor.execute(f'INSERT INTO {name} ( asset_id, data) VALUES (?, ?)', (asset_id, data))
        db.commit()
        db.close()
        return True

    def is_stored(self, name, asset_id):
        db, cursor = self._connect(name)
        cursor.execute(f'SELECT data FROM {name}  WHERE asset_id = ?', (asset_id,))
        row = cursor.fetchone()
        db.close()
        return row is not None

    def clear(self, name=None):
        if name:
            db, cursor = self._connect(name)
            cursor.execute(f'DROP TABLE {name}')
            db.commit()
            db.close()
        else:
            if os.path.exists(self._filename):
                os.remove(self._filename)
        self._table_names = []

    def _connect(self, name):
        if name not in self.ALLOWED_NAMES:
            raise ValueError(f'name {name} is not allowed')

        if name not in self._table_names:
            self._create_tables(name)
        db = sqlite3.connect(self._filename)
        cursor = db.cursor()
        return (db, cursor)

    def _create_tables(self, name):
        db = sqlite3.connect(self._filename)
        cursor = db.cursor()
        cursor.execute(f'SELECT name FROM sqlite_master WHERE type=? AND name=?', ('table', name))
        row = cursor.fetchone()
        if not row:
            cursor.execute(f'CREATE TABLE {name} (asset_id, data)')
            db.commit()
        db.close()

        self._table_names.append(name)

    @property
    def filename(self):
        return self._filename
