"""

 Invoke Agent API

"""

import json
import logging

from flask import (
    Blueprint,
    jsonify
)

logger = logging.getLogger('invokable.ui.agent')


def get_agent_blueprint(uri, title, description, api_version, ddo_text):
    agent_ui = Blueprint(uri, 'invokable.agent')

    @agent_ui.route('/ddo', methods=['GET'])
    def get_ddo():
        """
        Get the registered DDO for this agent
        ---
        tags:
          - agent api
        responses:
          200:
            description: return a json object of the agents DDO
          404:
            description: field value error
          500:
            description: database error
        """
        return jsonify(json.loads(ddo_text))

    @agent_ui.route('/status', methods=['GET'])
    def get_list():
        """
        Get agent status
        ---
        tags:
          - agent api
        responses:
          200:
            description: return JSON record of the agent status
          404:
            description: field value error
          500:
            description: database error
        """
        result = {
            'name': title,
            'description': description,
            'api-versions': [api_version],
        }
        return jsonify(result)

    return agent_ui
