"""

 Invoke Agent API

"""

import logging

from flask import (
    Blueprint,
    jsonify,
    request
)

logger = logging.getLogger('invokable.ui.agent')


def get_authentication_blueprint(uri, authentication):
    auth_ui = Blueprint(uri, 'invokable.authentication')

    @auth_ui.route('/token', methods=['GET'])
    def get_token():
        """
        Get an authorization token using a username and password
        ---
        tags:
          - authentication api
        responses:
          200:
            description: return a json object of the agents DDO
          404:
            description: field value error
          500:
            description: database error
        """
        auth = request.authorization
        if authentication.is_basic_authorized(auth):
            return jsonify(authentication.get_tokens(auth.username))

        return authentication.auth_error_callback()

    @auth_ui.route('/token', methods=['POST'])
    def add_token():
        """
        Create an authorization token using a username and password
        ---
        tags:
          - authentication api
        responses:
          200:
            description: return a json object of the agents DDO
          404:
            description: field value error
          500:
            description: database error
        """
        auth = request.authorization
        if authentication.is_basic_authorized(auth):
            return jsonify(authentication.create_token(auth.username))

        return authentication.auth_error_callback()

    @auth_ui.route('/revoke/<token>', methods=['POST', 'DELETE'])
    def revoke_token(token):
        """
        Create an authorization token using a username and password
        ---
        tags:
          - authentication api
        responses:
          200:
            description: return a json object of the agents DDO
          404:
            description: field value error
          500:
            description: database error
        """
        auth = request.authorization
        if authentication.is_basic_authorized(auth):
            result = authentication.revoke_token(auth.username, token)
            return jsonify(str(result))

        return authentication.auth_error_callback()

    return auth_ui
