"""

 Invoke Storage API

"""

import logging

from flask import (
    Blueprint,
    request
)

logger = logging.getLogger('invokable.ui.storage')

STORAGE_ASSET = 'asset'


def get_storage_blueprint(uri, storage, authentication):
    storage_ui = Blueprint(uri, 'invokable.storage')

    @storage_ui.route('<asset_id>', methods=['GET'])
    @authentication.login_maybe_required
    def get_asset(asset_id):
        """
        Download asset based on it's ID
        ---
        tags:
          - storage api
        parameters:
          - name: asset_id
            in: path
            description: asset given job_id.
            required: true
            type: string
        responses:
          200:
            description: return a json object of the job
          404:
            description: field value error
          500:
            description: database error
        """
        if storage.is_stored(STORAGE_ASSET, asset_id):
            asset_data = storage.read(STORAGE_ASSET, asset_id)
            if asset_data:
                return asset_data
        return 'asset not found', 404

    @storage_ui.route('<asset_id>', methods=['POST', 'PUT'])
    @authentication.login_maybe_required
    def set_asset(asset_id):
        """
        Upload asset
        ---
        tags:
          - storage api
        responses:
          200:
            description: return a list of jobs the jobs status.
          404:
            description: field value error
          500:
            description: database error
        """

        if request.data:
            if not storage.write(STORAGE_ASSET, asset_id, request.data):
                return 'unable to save asset data', 507
        else:
            return 'asset not registered', 404

        return asset_id

    return storage_ui
