"""

Conftest


"""

import pytest
import logging
from requests.auth import _basic_auth_str

from tests.libs.config import Config

from invokable import InvokableServer
from starfish.agent import RemoteAgent
from starfish.agent.services import Services

class SitePath():
    url = 'http://localhost:9090'
    meta = '/api/v1/meta'
    invoke = '/api/v1/invoke'
    agent = '/api'
    auth = '/api/v1/auth'
    storage = '/api/v1/assets'
    docs = '/api/vl/docs'

CONFIG_FILE = 'tests/resources/config_spree.conf'

class SiteInfo:
    username = 'Aladdin'
    password = 'OpenSesame'

logging.basicConfig(level=logging.DEBUG)
logging.getLogger('web3').setLevel(logging.INFO)
logging.getLogger('urllib3').setLevel(logging.INFO)

logger = logging.getLogger('invokable-py/test')

asset_data = {}
server_instances = {}

@pytest.fixture(scope='module')
def config():
    config = Config(CONFIG_FILE)
    return config

@pytest.fixture(scope='module')
def site_path():
    return SitePath()

@pytest.fixture(scope='module')
def site_info():
    return SiteInfo()

@pytest.fixture(scope='module')
def services(site_path):
    services = Services('')
    services.add('meta', site_path.meta)
    services.add('auth', site_path.auth)
    services.add('invoke', site_path.invoke)
    services.add('storage', site_path.storage)
    return services

def get_server_instance(users=None):
    global asset_data, server_instances
    site_path = SitePath()
    services = Services('')
    services.add('meta', site_path.meta)
    services.add('invoke', site_path.invoke)
    services.add('storage', site_path.storage)

    server_name = 'standard'
    if users:
        server_name = 'secure'
        services.add('auth', site_path.auth)
    if server_name not in server_instances:
        ddo = RemoteAgent.generate_ddo(services)
        server_instances[server_name] = InvokableServer(
            ddo_text = ddo.as_text(),
            users = users,
            storage = 'memory',
            data = {
                'asset_data': asset_data
            },
        )
    return server_instances[server_name]

@pytest.fixture(scope='module')
def server():
    return get_server_instance()


@pytest.fixture(scope='module')
def server_secure(site_info, services):
    users = {
        site_info.username: site_info.password
    }
    return get_server_instance(users)

@pytest.fixture(scope='module')
def client(server):
    server.add_package('tests.resources.services.asset')
    server.add_package('tests.resources.services.basic')
    app = server.create_app()
    app.config['TESTING'] = True
    return app.test_client()

@pytest.fixture(scope='module')
def client_secure(server_secure):
    server_secure.add_package('tests.resources.services.asset')
    server_secure.add_package('tests.resources.services.basic')
    app = server_secure.create_app()
    app.config['TESTING'] = True
    return app.test_client()

@pytest.fixture(scope='module')
def invoke_handler(server):
    return server.invoke_handler


@pytest.fixture(scope='module')
def auth_token(client_secure, site_path, site_info):
    url = f'{site_path.auth}/token'
    headers={
        'Authorization': _basic_auth_str(site_info.username, site_info.password)
    }
    response = client_secure.get(url, headers=headers)
    token_list = response.get_json()
    if len(token_list) > 0:
        token = token_list[-1]
    else:
        response = client_secure.post(url, headers=headers)
        token = response.get_json()

    return token

