"""

Conftest


"""

import pytest
import logging
import datetime


from starfish import DNetwork
from starfish.account import Account
from starfish.agent import RemoteAgent
from invokable import InvokableServer


TEST_LISTING_DATA = {
    'name': 'Test file asset',
    'description': 'Test asset for sale',
    'dateCreated': datetime.datetime.now().strftime('%Y-%m-%dT%H:%M:%SZ'),
    'creator': 'Test starfish',
    'license': 'Closed',
    'price': 3.141592,              # price is in ocean tokens
    'additionalInformation': 'Some extra data',
    'tags': ['asset', 'sale', 'test', 'starfish'],
}


logging.basicConfig(level=logging.DEBUG)
logging.getLogger('web3').setLevel(logging.INFO)
logging.getLogger('urllib3').setLevel(logging.INFO)

logger = logging.getLogger('invokable-py/test')

@pytest.fixture(scope='module')
def listing_data():
    return TEST_LISTING_DATA

@pytest.fixture(scope='module')
def network(config):
    network = DNetwork(config.network.url)
    return network

@pytest.fixture(scope='module')
def remote_agent_storage(config, site_info, network):

    agent_url = config.remote_agent.url
    authentication_access = {
        'username': site_info.username,
        'password': site_info.password,
    }
    return RemoteAgent.load(network, agent_url, authentication_access)

@pytest.fixture(scope='module')
def remote_agent_invoke(config, site_info, network, services, auth_token):

    # the starfish-py middleware uses auth option in http get, which is not supported
    # in the mock http server. So we need to get the auth_token using our own
    # special get

    agent_url = config.remote_agent.url
    authentication_access = {
        'username': site_info.username,
        'password': site_info.password,
        'token': auth_token
    }
    """
    ddo = RemoteAgent.generate_ddo(services)
    register_account = Account(config.publisher_account)
    return RemoteAgent.register(network, register_account, ddo, authentication_access)
    """
    agent = RemoteAgent.load(network, agent_url, authentication_access)
    return agent
