"""

Config


"""
import re
import os

from configparser import (
    ConfigParser,
    ExtendedInterpolation,
)


class ConfigSection():
    @property
    def as_dict(self):
        return self.__dict__

    def __len__(self):
        return len(self.__dict__)

    def __getitem__(self, name):
        return self.__dict__[name]

    def keys(self):
        return self.__dict__.keys()

class Config:
    _instance = None

    CONFIG_DEFINE = {
        'main': {
            'port': 8090,
            'server_url': 'http://localhost:9090',
        },
        'network': {
            'url': None,
        },
        'publisher account': {
            'address': [None, True],
            'password': [None, True],
            'keyfile': [None, True],
        },
        'purchaser account': {
            'address': [None, True],
            'password': [None, True],
            'keyfile': [None, True],
        },
        'agent account': {
            'address': [None, True],
            'password': [None, True],
            'keyfile': [None, True],
        },
        'remote agent': {
            'url': 'http://localhost:8080',
            'username': None,
            'password': None,
        },
    }


    def __init__(self, filename=None):
        if filename:
            self.read(filename)

    def read(self, filename=None):
        config = ConfigParser(interpolation=ExtendedInterpolation())
        if filename:
            if os.path.exists(filename):
                config.read(filename)
            else:
                raise FileNotFoundError(f'cannot find config file "{filename}"')

        for section_name, values in Config.CONFIG_DEFINE.items():
            object_name = re.sub(r'\s', '_', section_name)
            config_section = ConfigSection()
            setattr(self, object_name, config_section)
            if isinstance(values, str):
                getattr(self, values)(config, section_name, config_section)
            elif isinstance(values, dict):
                for name, default_value in values.items():
                    is_validate = False
                    if isinstance(default_value, list):
                        is_validate = default_value[1]
                        default_value = default_value[0]

                    value = config.get(section_name, name, fallback=default_value)
                    if value is None and is_validate:
                        raise ValueError(f'Config value must be completed [{section_name}] "{name}"')
                    setattr(config_section, name, value)
            else:
                raise TypeError('Unknown config filter type')

    @staticmethod
    def get_instance():
        if not Config._instance:
            Config._instance = Config()
        return Config._instance
