"""

Invokable Service asset_acknowledge

"""

import secrets
from invokable import InvokeService


class AssetAcknowledge(InvokeService):

    name = 'asset_acknowledge'
    description = 'Acknowledge an asset registered on the Ocean Network'
    creator = 'Dex'
    modes = InvokeService.MODES_ALL
    inputs = {
        'did': {
            'type': 'string',
            'required': True
        },
        'recipient_name': {
            'type': 'array',
            'items': {
                'type': 'string'
            }
        }
    }
    outputs = {
        'acknowledge_list': {
            'type': 'array',
            'items': {
                'type': 'string'
            }
        }
    }
    tags = ['asset', 'acknowledge']


    def run(self, inputs, data=None):
        asset_data = data['asset_data']

        did = inputs['did']
        recipient_name = inputs['recipient_name']

        ack_list = {}
        if did in asset_data:
            ack_id = secrets.token_hex(16)
            ack_did = f'did:dep:{ack_id}'
            asset_data[did]['ack_id'] = ack_id
            ack_list[ack_id] = asset_data[did]['asset_id']

            return {
                'acknowledge_list': ack_list,
            }
        return f'{did} not found', 404
