#!/usr/bin/env python3
"""

Server test

"""

import os
import argparse
from tests.libs.config import Config
from invokable import InvokableServer


WEB_SERVER_PORT=9090
CONFIG_FILENAME='tests/resources/config_spree.conf'

asset_data = {}

def main():


    parser = argparse.ArgumentParser(description='Manually test Invokable Web Server')

    parser.add_argument(
        '-p',
        '--port',
        help=f'web server port, default: {WEB_SERVER_PORT}',
        default=WEB_SERVER_PORT
    )

    parser.add_argument(
        '-c',
        '--config',
        help=f'config file, default: {CONFIG_FILENAME}',
        default=CONFIG_FILENAME
    )

    parser.add_argument(
        '-d',
        '--debug',
        action='store_true',
        help=f'Debug mode on or off, default: off',
    )

    parser.add_argument(
        '-u',
        '--user',
        help='username',
    )

    parser.add_argument(
        '--password',
        help='password',
    )

    args = parser.parse_args()

    config_data = Config()
    config_data.read(args.config)

    users = None
    if args.user and args.password:
        users = {
            args.user: args.password
        }

    server = InvokableServer(
        port=args.port,
        package = [
            'tests.resources.services.asset',
            'tests.resources.services.basic',
        ],
        users = users,
        data={
            'asset_data':asset_data
        }
    )
    app = server.create_app('test_invokable_server')

    app.debug = args.debug
    app.run(host=server.host, port=server.port)

if __name__ == '__main__':
    main()
