"""

 Test Invoke Server

"""

import pytest
import secrets
import datetime
import json
import re

# from server.api.users import get_user_list

from invokable import InvokableServer

TEST_HOST = 'test1234'
TEST_PORT = 9999

def _get_swagger_spec(app):
    app.config['TESTING'] = True
    client = app.test_client()
    url = '/spec'
    response = client.get(url)
    result = response.get_json()
    assert(result)
    return result


def _assert_in_path_list(path_list, values):
    for name in values:
        assert(name in path_list)

def _assert_not_in_path_list(path_list, values):
    for name in values:
        assert(name not in path_list)

def test_server_basic(site_path):

    server = InvokableServer(
        host=TEST_HOST,
        port=TEST_PORT,
        package = [
            'tests.resources.services.asset',
            'tests.resources.services.basic'
        ]

    )
    assert(server.host == TEST_HOST)
    assert(server.port == TEST_PORT)
    assert(server.url == f'http://{TEST_HOST}:{TEST_PORT}')
    app = server.create_app()
    result = _get_swagger_spec(app)
    _assert_in_path_list(result['paths'], [f'{site_path.invoke}/jobs',
        f'{site_path.meta}/data',
        f'{site_path.invoke}/sync/asset_list',
        f'{site_path.invoke}/sync/hash_text']
    )

def test_server_swagger_filter_string(site_path):

    server = InvokableServer(
        host=TEST_HOST,
        port=TEST_PORT,
        package = [
            'tests.resources.services.asset',
            'tests.resources.services.basic'
        ],
        swagger_list = 'asset_list'
    )
    app = server.create_app()
    result = _get_swagger_spec(app)

    _assert_in_path_list(result['paths'], [f'{site_path.invoke}/jobs',
        f'{site_path.meta}/data',
        f'{site_path.invoke}/sync/asset_list']
    )
    _assert_not_in_path_list(result['paths'], [f'{site_path.invoke}/hash_text'])


def test_server_swagger_filter_list(site_path):

    server = InvokableServer(
        host=TEST_HOST,
        port=TEST_PORT,
        package = [
            'tests.resources.services.asset',
            'tests.resources.services.basic'
        ],
        swagger_list = ['asset_list', 'asset_info_get']
    )
    app = server.create_app()
    result = _get_swagger_spec(app)

    _assert_in_path_list(result['paths'], [f'{site_path.invoke}/jobs',
        f'{site_path.meta}/data',
        f'{site_path.invoke}/sync/asset_list', f'{site_path.invoke}/sync/asset_info_get',
        f'{site_path.invoke}/async/asset_list', f'{site_path.invoke}/async/asset_info_get']
    )
    _assert_not_in_path_list(result['paths'], ['/invoke/hash_text'])


def test_server_swagger_filter_type(site_path):

    server = InvokableServer(
        host=TEST_HOST,
        port=TEST_PORT,
        package = [
            'tests.resources.services.asset',
            'tests.resources.services.basic'
        ],
        swagger_list = ['asset_list', 'asset_info_get'],
        swagger_list_type = 'sync'
    )
    app = server.create_app()
    result = _get_swagger_spec(app)

    _assert_in_path_list(result['paths'], [f'{site_path.invoke}/jobs',
        f'{site_path.meta}/data',
        f'{site_path.invoke}/sync/asset_list',
        f'{site_path.invoke}/sync/asset_info_get']
    )
    print(result['paths'])

    _assert_not_in_path_list(result['paths'], [f'{site_path.invoke}/sync/hash_text',
        f'{site_path.invoke}/async/asset_list',
        f'{site_path.invoke}/async/asset_info_get']
    )

def test_server_authentication(site_path):

    server = InvokableServer(
        host=TEST_HOST,
        port=TEST_PORT,
        package = [
            'tests.resources.services.asset',
            'tests.resources.services.basic'
        ],
        users= {'Aladdin': 'OpenSesame'}

    )
    assert(server.host == TEST_HOST)
    assert(server.port == TEST_PORT)
    assert(server.url == f'http://{TEST_HOST}:{TEST_PORT}')
    app = server.create_app()
    app.config['TESTING'] = True
    client = app.test_client()
    url = site_path.meta
    response = client.get(url)
    result = response.get_json()
    print(result)

