"""

 Test User API

"""

import pytest
import secrets
import datetime
import json
import re
import time

# from server.api.users import get_user_list

from invokable import JobStatus
from invokable import InvokeService

asset_data = {}

def _register_asset(client, inputs, invoke_handler):

    result = invoke_handler.invoke('asset_register', inputs=inputs, data={'asset_data': asset_data})

    assert(result)
    return result['outputs']['listing_did']

def test_register_asset(client, invoke_handler):
    inputs = {
        'name': 'test INR name',
        'ref_id': secrets.token_hex(64),
    }
    did = _register_asset(client, inputs, invoke_handler)
    assert(did)
    assert(re.match('^did:dep:[0-9a-fA-F]+', did))

def test_get_asset_list(client, invoke_handler):

    inputs = None
    result = invoke_handler.invoke('asset_list', inputs=inputs, data={'asset_data': asset_data})
    assert(result)
    assert(result['status'] == JobStatus.SUCCEEDED)
    assert(result['outputs']['items'])

def test_get_asset_info(client, invoke_handler):

    inputs = {
        'name': 'test INR name',
        'ref_id': secrets.token_hex(64),
    }
    did = _register_asset(client, inputs, invoke_handler)

    inputs = {
        'did': did
    }
    result = invoke_handler.invoke('asset_info_get', inputs=inputs, data={'asset_data': asset_data})
    assert(result)
    assert(result['outputs']['data'])
    assert(result['outputs']['extra_data'])

def test_get_async_asset_list(client, invoke_handler):

    inputs = None
    result = invoke_handler.invoke('asset_list', inputs=inputs, is_async=True, data={'asset_data': asset_data})
    assert(result)
    assert(result['job-id'])
    job_id = result['job-id']
    job_list = invoke_handler.get_job_list()
    assert(job_list[job_id])
    assert(job_list[job_id] == JobStatus.RUNNING)
    while True:
        status = invoke_handler.get_job_status(job_id)
        if invoke_handler.is_job_done(job_id):
             break
        time.sleep(1)
    result = invoke_handler.get_job_result(job_id)
    assert(result['status'] == JobStatus.SUCCEEDED)
    assert(result['outputs']['items'])

def get_test_asset_list(service, inputs=None, asset_data=None):

    if inputs and 'test' in inputs:
        return {'items': asset_data}
    return 'bad inputs', 800

def test_get_asset_list_function(client, invoke_handler):
    asset_data = []
    for i in range(0, 100):
        asset_data.append(secrets.token_hex(16))
    invoke_handler.register_function(
        'test_asset_list_function',
        get_test_asset_list,
        inputs = {
            'test': {
                'type': 'string',
                'required': False
            }
        },
        outputs = {
            'items': {
                'type': 'array',
                'items' : {
                    'type': 'string'
                }
            }
        }
    )

    inputs = {'test': '12345'}
    result = invoke_handler.invoke('test_asset_list_function', inputs=inputs, data={'asset_data':asset_data})
    assert(result)
    assert(result['status'] == JobStatus.SUCCEEDED)
    assert(result['outputs']['items'])

    result = invoke_handler.invoke('test_asset_list_function', data={'asset_data':asset_data})
    assert(result)
    assert(result['status'] == JobStatus.FAILED)
    assert(result['outputs']['description'] == 'bad inputs')
    assert(result['outputs']['errorcode'] == 800)
