"""

 Test Invoke Register

"""

import pytest
import secrets
import datetime
import json
import re


def test_register_no_seperatore(invoke_handler):
    text = """
Some description
"""

    with pytest.raises(SyntaxError):
        invoke_handler.register(text)

def test_register_invalid_data_type(invoke_handler):
    text = ["Some description"]

    with pytest.raises(TypeError):
        invoke_handler.register(text)

def test_register_simple(invoke_handler):
    text = """

Test Service description

---
name: test_1
creator: Dex
modes:
  - sync
  - async
tags:
  - hash
  - text
inputs:
  - name: to_hash
    required: true
    description: item to hash
    type: string

outputs:
  - name: hash_value
    type: string
    description: return hash value of to_hash
  - name: hash_data
    type: asset
    description: asset data

"""

    invoke_handler.register(text)

def test_register_complex_params(invoke_handler):
    text = """

Test Service description
with multiple lines

---
name: test_2
creator: Dex
modes:
  - sync
  - async
tags:
  - hash
  - text
inputs:
  to_hash:
    position: 2
    required: true
    description: item to hash
    type: string

  param_1:
    position: 0
    description: complex object
    type: object
    properties:
      id:
        type: integer
      name:
        type: text

  param_2:
    position: 1
    description: array of objects
    type: array
    items:
      type: object
      properties:
        id:
          type: integer
        name:
          type: text

outputs:
  hash_value:
    description: return hash value of to_hash
    type: object
    properties:
      id:
        type: integer
      name:
        type: text
  asset_data:
    type: asset

"""
    invoke_handler.register(text)
