"""

 Test Invoke Metadata generation

"""

import pytest
import secrets
import datetime
import json
import re

# from server.api.users import get_user_list

from invokable import InvokeService


class TestService(InvokeService):

    name = 'asset_register'
    description = 'Register an asset on the Ocean Network'
    creator = 'Dex'
    modes = InvokeService.MODES_ALL
    inputs = {
        'ref_id': {
            'type': 'string',
            'required': True
        },
        'asset_list': {
            'type': 'array',
            'items': {
                'type': 'string'
            }
        }
    }
    outputs = {
        'listing_did': {
            'type': 'string'
        }
    }
    tags = ['asset', 'register']

    def run(self, inputs, config):
        pass

def asset_tree_value(data, tree_list, value):
    walk = data
    for item in tree_list:
        assert(walk[item])
        if isinstance(walk[item], (dict, tuple, list, set)):
            walk = walk[item]
    assert(walk[item] == value)

def test_metadata_generate():
    test = TestService()
    result = test.generate_metadata()
    assert(result)
    data = json.loads(result)
    assert(data['name'] == 'asset_register')
    asset_tree_value(data, ['operation','inputs', 'ref_id', 'type'], 'json')

    asset_string_format_list = ['did', 'did_asset', 'asset']
    for format_type in asset_string_format_list:
        test.inputs = {
            'asset_did': {
                'type': 'asset',
                'format': format_type,
                'required': True
            },
        }
        result = test.generate_metadata()
        assert(result)
        data = json.loads(result)

        asset_tree_value(data, ['operation','inputs', 'asset_did', 'type'], 'asset')


def test_metadata_name():
    del(TestService.name)
    test = TestService()
    result = test.generate_metadata()
    assert(result)
    data = json.loads(result)
    assert(data['name'] == 'test_service')


def test_metadat_description():
    del(TestService.description)
    test = TestService()
    result = test.generate_metadata()
    assert(result)
    data = json.loads(result)
    assert(not 'description' in data)


def test_metadata_modes():
    del(TestService.modes)
    TestService.modes = InvokeService.MODE_ASYNC
    test = TestService()
    result = test.generate_metadata()
    assert(result)
    data = json.loads(result)
    assert(data['operation']['modes'] == ['async'])

    del(TestService.modes)
    test = TestService()
    result = test.generate_metadata()
    assert(result)
    data = json.loads(result)
    assert(data['operation']['modes'] == InvokeService.MODES_ALL)

    TestService.modes = InvokeService.MODES_ALL
    test = TestService()
    result = test.generate_metadata()
    assert(result)
    data = json.loads(result)
    assert(data['operation']['modes'] == InvokeService.MODES_ALL)
