"""

 Test Operation metadata

"""

import pytest

from jsonschema.exceptions import ValidationError
from invokable.metadata_tools import metadata_validator


def test_metadata_validator_simple():
    metadata = {
        'name': 'string',
        'type': 'operation',
        'description': 'string',
        'dateCreated': 'string',
        'creator': 'string',
        'license': 'string',
        'copyrightHolder': 'string',
        'operation': {
            'inputs':  None,
            'outputs': None
        }
    }
    assert(metadata_validator(metadata))

    metadata['extra-name'] = 'invalid field'
    with pytest.raises(ValidationError, match=r'Additional properties are not allowed'):
        assert(metadata_validator(metadata))

    del metadata['extra-name']

    metadata['additionalInformation'] = 'string'
    with pytest.raises(ValidationError):
        assert(metadata_validator(metadata))

    metadata['additionalInformation'] = ['string 1', 'string 2']
    assert(metadata_validator(metadata))

    metadata['additionalInformation'] = {
        'operation': {
            'version': '1.0.0',
            'inputs': {
                'field_1': { 'type': 'string' },
                'field_2': { 'type': 'string' },
            },
            'outputs': {
                'asset_id': { 'type': 'asset'}
            }
        }
    }

    assert(metadata_validator(metadata))

    metadata['operation']['inputs'] = {
        'arg_1': {
            'type': 'asset',
            'position': 1,
            'required': True
        }
    }

    assert(metadata_validator(metadata))

    # modes not an array
    metadata['operation']['modes'] = 'sync'
    with pytest.raises(ValidationError,match=r"'sync' is not of type 'array'"):
        assert(metadata_validator(metadata))

    metadata['operation']['modes'] = ['sync']
    assert(metadata_validator(metadata))

    metadata['operation']['modes'] = ['bad-sync']
    with pytest.raises(ValidationError, match=r"'bad-sync' is not one of"):
        assert(metadata_validator(metadata))

    metadata['operation']['modes'] = ['async', 'sync']
    metadata['operation']['inputs']['arg_1']['type'] = 'bad-type'
    with pytest.raises(ValidationError, match=r"'bad-type' is not one of"):
        assert(metadata_validator(metadata))

    metadata['operation']['inputs'] = {
        'arg_1': {
            'position': 1,
            'required': True
        }
    }
    with pytest.raises(ValidationError, match=r"'type' is a required property"):
        assert(metadata_validator(metadata))

    metadata['operation']['inputs'] = {
        'arg_1': 'string'
    }
    with pytest.raises(ValidationError, match=r"'string' is not of type 'object'"):
        assert(metadata_validator(metadata))

    metadata['operation']['inputs'] = {
        'arg_1': {}
    }
    with pytest.raises(ValidationError, match=r"'type' is a required property"):
        assert(metadata_validator(metadata))
