"""

 Test Storage Objects

"""

import pytest
import os
import secrets

# from server.api.users import get_user_list

from invokable.storage import storage_load
from invokable.storage.storage_memory import StorageMemory
from invokable.storage.storage_sqlite import StorageSqlite

TEMP_DB_FILENAME = '/tmp/test.db'
STORAGE_ITEM_COUNT = 100
STORAGE_NAME_METADATA = 'metadata'
STORAGE_NAME_ASSET = 'asset'

def _test_storage_object(name, store, count):
    items = {}
    # test write/insert
    for i in range(0, count):
        asset_id = secrets.token_hex(64)
        data = secrets.token_hex(128)
        items[asset_id] = data
        store.write(name, asset_id, data)

    # test write/update
    for asset_id, data in items.items():
        new_data = secrets.token_hex(128)
        items[asset_id] = new_data
        store.write(name, asset_id, new_data)

    # test is_stored & read
    for asset_id, data in items.items():
        assert(store.is_stored(name, asset_id))
        read_data = store.read(name, asset_id)
        assert(read_data == data)

    # test get all items
    read_items = store.items(name)
    for asset_id, data in items.items():
        assert(items[asset_id] == data)


def test_memory_storage():

    store = storage_load('memory')
    assert(store)
    assert(isinstance(store, StorageMemory))
    _test_storage_object(STORAGE_NAME_METADATA, store, STORAGE_ITEM_COUNT)

    store.clear()
    assert(store.items(STORAGE_NAME_METADATA) == {})

def test_sqlite_storage():

    if os.path.exists(TEMP_DB_FILENAME):
        os.remove(TEMP_DB_FILENAME)
    store = storage_load(f'sqlite:{TEMP_DB_FILENAME}')
    assert(store)
    assert(isinstance(store, StorageSqlite))
    _test_storage_object(STORAGE_NAME_METADATA, store, STORAGE_ITEM_COUNT)
    _test_storage_object(STORAGE_NAME_ASSET, store, STORAGE_ITEM_COUNT)
    store.clear(STORAGE_NAME_ASSET)
    assert(store.items(STORAGE_NAME_ASSET) == {})
    assert(os.path.exists(TEMP_DB_FILENAME))

    store.clear()
    assert(not os.path.exists(TEMP_DB_FILENAME))

