"""

 Test Authentication API

"""

import pytest
import requests
from requests.auth import _basic_auth_str


def get_token_list(client_secure, site_path, site_info):
    url = f'{site_path.auth}/token'
    headers={
        'Authorization': _basic_auth_str(site_info.username, site_info.password)
    }
    response = client_secure.get(url, headers=headers)
    result = response.get_json()
    assert(isinstance(result, list))
    return result

def test_get_add_token(config, client_secure, site_path, site_info):

    surfer_username = config.remote_agent.username
    surfer_password = config.remote_agent.password
    surfer_url = config.remote_agent.url

    token_url = surfer_url + '/api/v1/auth/token'
    response = requests.get(token_url, auth=(surfer_username, surfer_password))
    result = response.json()
    assert(isinstance(result, list))

    token_list = get_token_list(client_secure, site_path, site_info)

    url = f'{site_path.auth}/token'
    headers={
        'Authorization': _basic_auth_str(site_info.username, site_info.password)
    }
    response = client_secure.post(url, headers=headers)
    add_token = response.get_json()

    assert(add_token)

    token_list = get_token_list(client_secure, site_path, site_info)
    assert(add_token in token_list)

def test_revoke_token(client_secure, site_path, site_info):
    token_list = get_token_list(client_secure, site_path, site_info)
    token = token_list[-1]

    url = f'{site_path.auth}/revoke/{token}'
    headers={
        'Authorization': _basic_auth_str(site_info.username, site_info.password)
    }
    response = client_secure.post(url, headers=headers)
    result = response.get_json()
    assert(result == 'True')


