"""

 Test User API

"""

import pytest
import secrets
import datetime
import json
import re

from invokable import JobStatus


def _register_asset(client, inputs, site_path):
    url = f'{site_path.invoke}/sync/asset_register'
    response = client.post(url, json=inputs)
    result = response.get_json()
    assert(result)
    return result['outputs']['listing_did']

def test_register_asset(client, site_path):
    inputs = {
        'name': 'test INR name',
        'ref_id': secrets.token_hex(64),
    }
    did = _register_asset(client, inputs, site_path)
    assert(did)
    assert(re.match('^did:dep:[0-9a-fA-F]+', did))

def test_get_asset_list(client, site_path):

    inputs = {}
    url = f'{site_path.invoke}/sync/asset_list'
    response = client.post(url, json=inputs)
    result = response.get_json()
    assert(result)
    assert(result['status'] == JobStatus.SUCCEEDED)
    assert(result['outputs']['items'])

def test_get_asset_info(client, site_path):

    inputs = {
        'name': 'test INR name',
        'ref_id': secrets.token_hex(64),
    }
    did = _register_asset(client, inputs, site_path)

    inputs = {
        'did': did
    }
    url = f'{site_path.invoke}/sync/asset_info_get'
    response = client.post(url, json=inputs)
    result = response.get_json()
    assert(result)
    assert(result['outputs']['data'])
    assert(result['outputs']['extra_data'])



def test_create_job(client, site_path):
    inputs = {}
    url = f'{site_path.invoke}/async/asset_list'
    response = client.post(url, json=inputs)
    result = response.get_json()
    assert(result)
    assert(result['job-id'] )

def test_get_job_list(client, site_path):

    url = f'{site_path.invoke}/jobs'
    response = client.get(url)
    result = response.get_json()
    assert(result)

def test_get_job_item(client, site_path):
    url = f'{site_path.invoke}/jobs'
    response = client.get(url)
    result = response.get_json()
    for job_id, status in result.items():
        url = f'{site_path.invoke}/jobs/{job_id}'
        response = client.get(url)
        data = response.get_json()
        assert(data)
        assert(data['status'] == JobStatus.SUCCEEDED
                or data['status'] == JobStatus.RUNNING
                or data['status'] == JobStatus.FAILED)
