"""

 Test Swagger UI

"""

import pytest
import secrets
import datetime
import json
import re

from invokable import JobStatus
from invokable import InvokableServer



def assert_parameters(params, inputs):
    assert(params)
    assert(inputs)

def assert_results(responses, outputs):
    assert(responses)
    assert(responses['200'])
    assert(responses['200']['schema']['properties'])

def test_swagger_ui(site_path):
    # since we are testing after a few services have changed in this test suite
    # we now need to load a new server with a clean set of services to test against

    server = InvokableServer()
    server.add_package('tests.resources.services.asset')
    server.add_package('tests.resources.services.basic')
    app = server.create_app()
    app.config['TESTING'] = True
    client = app.test_client()

    url = '/spec'
    response = client.get(url)
    result = response.get_json()

    asset_list = server.invoke_handler.asset_list
    for asset_id, invoke_item in asset_list.items():
        invoke_name = f'/api/v1/invoke/sync/{asset_id}'
        assert(invoke_name in result['paths'])
        invoke_spec = result['paths'][invoke_name]
        if 'inputs' in invoke_item.metadata_additional and invoke_item.metadata_additional['inputs']:
            assert_parameters(invoke_spec['parameters'], invoke_item.metadata_additional['inputs'])

        if 'outputs' in invoke_item.metadata_additional and  invoke_item.metadata_additional['outputs']:
            assert_results(invoke_spec['post']['responses'], invoke_item.metadata_additional['outputs'])
